==> GotoBLAS-1.15.tar.gz: GotoBLAS/lapack/potrf/potrf_U_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

static FLOAT dm1 = -1.;

#ifndef COMPLEX
#define TRSM_KERNEL   TRSM_KERNEL_LT
#else
#define TRSM_KERNEL   TRSM_KERNEL_LC
#undef  SYRK_KERNEL_U
#ifdef XDOUBLE
#define SYRK_KERNEL_U xherk_kernel_UC
#elif defined(DOUBLE)
#define SYRK_KERNEL_U zherk_kernel_UC
#else
#define SYRK_KERNEL_U cherk_kernel_UC
#endif
#endif

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)

blasint CNAME(long  n, FLOAT *a, long lda, FLOAT *sa, FLOAT *sb){

  long info;
  long bm, bk, i, blocking;
  long is, min_i;
  long ls, min_l;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  FLOAT *sub = (FLOAT *)alloca(GEMM_UNROLL_M * GEMM_UNROLL_N * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif
  
  if (n <= DTB_ENTRIES) {
    info = POTF2_U(n, a, lda, sb);
    return info;
  }

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;

  for (i = 0; i < n; i += blocking) {
    
    bk = n - i;
    if (bk > blocking) bk = blocking;

    info = CNAME(bk, a, lda, sa, sb);
    if (info) return info + i;
    
    bm = n - i - bk;
     
    if (bm > 0) {
      
      TRSM_IUNCOPY(bk, bk, a, lda, 0, sb);
      
      for(ls = 0; ls < bm; ls += REAL_GEMM_R){
	min_l = bm - ls;
	if (min_l > REAL_GEMM_R) min_l = REAL_GEMM_R;

	GEMM_ONCOPY (bk, min_l, a + (ls + bk) * lda * COMPSIZE, lda, sb2);
	
	for (is = 0; is < bk; is += GEMM_P) {
	  min_i = bk - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;

	  TRSM_KERNEL (min_i, min_l, bk, dm1, 
#ifdef COMPLEX
		       ZERO,
#endif
		       sb + bk * is * COMPSIZE, sb2,
		       a + (is + (ls + bk) * lda) * COMPSIZE, lda, is);
	}
	
	for(is = 0; is < ls + min_l; is += GEMM_P){
	  min_i = ls + min_l - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
#if  (GEMM_UNROLL_M == GEMM_UNROLL_N)
	  if (is < ls) {
	    GEMM_INCOPY(bk, min_i, a + (is + bk) * lda * COMPSIZE, lda, sa);
	    aa = sa;
	  } else {
	    aa = sb2 + bk * (is - ls) * COMPSIZE;
	  }
#else
	  GEMM_INCOPY(bk, min_i, a + (is + bk) * lda * COMPSIZE, lda, sa);
#endif

	  SYRK_KERNEL_U(min_i, min_l, bk, dm1,
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			aa, 
#else
			sa, 
#endif
			sb2,
			a + (is + bk + (ls + bk) * lda) * COMPSIZE, lda,
			is - ls, sub);
	}
      }
    }

    a += (1 + lda) * blocking * COMPSIZE;
  }
  
  return 0;
}

==> GotoBLAS-1.19.tar.gz: GotoBLAS/lapack/potrf/potrf_U_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

static FLOAT dm1 = -1.;

#ifndef COMPLEX
#define TRSM_KERNEL   TRSM_KERNEL_LT
#else
#define TRSM_KERNEL   TRSM_KERNEL_LC
#undef  SYRK_KERNEL_U
#ifdef XDOUBLE
#define SYRK_KERNEL_U xherk_kernel_UC
#elif defined(DOUBLE)
#define SYRK_KERNEL_U zherk_kernel_UC
#else
#define SYRK_KERNEL_U cherk_kernel_UC
#endif
#endif

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)
#define MAX_MN	MAX(GEMM_UNROLL_M, GEMM_UNROLL_N)

blasint CNAME(long  n, FLOAT *a, long lda, FLOAT *sa, FLOAT *sb){

  long info;
  long bm, bk, i, blocking;
  long is, min_i;
  long ls, min_l;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  FLOAT *sub = (FLOAT *)alloca(MAX_MN * MAX_MN * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif
  
  if (n <= DTB_ENTRIES) {
    info = POTF2_U(n, a, lda, sb);
    return info;
  }

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;

  for (i = 0; i < n; i += blocking) {
    
    bk = n - i;
    if (bk > blocking) bk = blocking;

    info = CNAME(bk, a, lda, sa, sb);
    if (info) return info + i;
    
    bm = n - i - bk;
     
    if (bm > 0) {
      
      TRSM_IUNCOPY(bk, bk, a, lda, 0, sb);
      
      for(ls = 0; ls < bm; ls += REAL_GEMM_R){
	min_l = bm - ls;
	if (min_l > REAL_GEMM_R) min_l = REAL_GEMM_R;

	GEMM_ONCOPY (bk, min_l, a + (ls + bk) * lda * COMPSIZE, lda, sb2);
	
	for (is = 0; is < bk; is += GEMM_P) {
	  min_i = bk - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;

	  TRSM_KERNEL (min_i, min_l, bk, dm1, 
#ifdef COMPLEX
		       ZERO,
#endif
		       sb + bk * is * COMPSIZE, sb2,
		       a + (is + (ls + bk) * lda) * COMPSIZE, lda, is);
	}
	
	for(is = 0; is < ls + min_l; is += GEMM_P){
	  min_i = ls + min_l - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
#if  (GEMM_UNROLL_M == GEMM_UNROLL_N)
	  if (is < ls) {
	    GEMM_INCOPY(bk, min_i, a + (is + bk) * lda * COMPSIZE, lda, sa);
	    aa = sa;
	  } else {
	    aa = sb2 + bk * (is - ls) * COMPSIZE;
	  }
#else
	  GEMM_INCOPY(bk, min_i, a + (is + bk) * lda * COMPSIZE, lda, sa);
#endif

	  SYRK_KERNEL_U(min_i, min_l, bk, dm1,
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			aa, 
#else
			sa, 
#endif
			sb2,
			a + (is + bk + (ls + bk) * lda) * COMPSIZE, lda,
			is - ls, sub);
	}
      }
    }

    a += (1 + lda) * blocking * COMPSIZE;
  }
  
  return 0;
}
